

#include "System.h"
#include "Utility.h"
#include "Params.h"
#include "SNPData.h"


Params params(false);

int debug = 0;
string input_dir, output_dir;
string input_file, snpprofile_file, output_file;
bool diplo;


string required_param[] =
{
  "input_file",
  "snpprofile_file",
  "output_file"
};
int num_required_param = sizeof(required_param) / sizeof(required_param[0]);

void InitParams()
{
  params.ExpandImports();

  srand(TimerSeconds());
  if (params.Contains("rand_seed"))
    srand(params["rand_seed"].GetInt());

  params.Require(vector<string>(&required_param[0],
                                &required_param[num_required_param]));

  if (params.Contains("work_dir"))
    input_dir = output_dir = params["work_dir"].GetString();

  if (params.Contains("input_dir"))
    input_dir = params["input_dir"].GetString();

  if (params.Contains("snpprofile_file"))
  {
    snpprofile_file = params["snpprofile_file"].GetString();
    string filetype = RemoveCompressedExtension(snpprofile_file);
    Assert(GetExtension(filetype) == "bin");
    filetype = GetExtension(RemoveExtension(filetype));
    Assert((filetype == "snpprofile") || (filetype == "snpprofiledata"));
  }

  if (params.Contains("input_file"))
  {
    input_file = params["input_file"].GetString();
    string filetype = RemoveCompressedExtension(input_file);
    Assert(GetExtension(filetype) == "bin");
    filetype = GetExtension(RemoveExtension(filetype));
    Assert((filetype == "haplopopdata") || (filetype == "diplopopdata"));
    diplo = (filetype == "diplopopdata");
  }

  if (params.Contains("output_dir"))
    output_dir = params["output_dir"].GetString();

  if (params.Contains("output_file"))
    output_file = params["output_file"].GetString();

  if (!input_dir.empty())
    cerr << "- Input directory = " << input_dir << endl;

  cerr << "- Input file = " << input_file << endl;
  cerr << "- SNPProfile file = " << snpprofile_file << endl;

  if (!output_dir.empty())
    cerr << "- Output directory = " << output_dir << endl;

  cerr << "- Output file = " << output_file << endl;
}


char GetChar(int b)
{
  if (b < 4)
    return base2char[b];
  else
    return '-';
}


int Main(vector<string> args)
{
  cerr << "ExportPopData" << endl;
  InitBaseTables();
  params.ImportArgs(vector<string>(args.begin() + 1, args.end()));
  InitParams();
  cerr << endl;

  SNPProfile profile;

  if (!snpprofile_file.empty())
  {
    string pathname;
    RelPath(input_dir, snpprofile_file, pathname);
    cerr << "* Reading SNPProfile file " << pathname << " ...";
    
    istream *in = InFileStream(pathname);
    AssertMsg(in, pathname);
    profile.ReadBin(*in);
    delete in;
    
    cerr << " done." << endl;
    profile.ComputeDerived();
    profile.Validate();
  }

  vector< vector<int8> > pops;

  string pathname;
  RelPath(input_dir, input_file, pathname);
  cerr << "* Reading data from " << pathname << " ...";
  istream *in = InFileStream(pathname);
  AssertMsg(in, pathname);

  ReadBin(*in, pops);
  for (int i = 0; i < pops.size(); ++i)
    Assert(pops[i].size() == profile.seq_len);

  delete in;
  cerr << " done." << endl;

  RelPath(output_dir, output_file, pathname);
  cerr << "* Writing file " << pathname << " ...";
  ostream *out = OutFileStream(pathname);
  AssertMsg(out, pathname);

  for (int i = 0; i < profile.seq_len; ++i)
  {
    if (profile.loci[i].label.empty())
      *out << profile.chroms[profile.loci[i].chrom].label << "." << profile.loci[i].genomic_pos;
    else
      *out << profile.loci[i].label;

    for (int k = 0; k < pops.size(); ++k)
    {
      if ((k & 1) == 0)
        *out << "\t";
      else
        *out << (diplo ? " " : "\t");
      *out << int(pops[k][i]);
    }

    *out << endl;
  }

  delete out;
  cerr << " done." << endl;
  
  cerr << "Done!" << endl;
  return 0;
}
