
#include "System.h"
#include "Utility.h"
#include "Params.h"
#include "SNPData.h"

Params params(false);

int debug = 0;
string hapmap_dir = "chrom/hg17/hapmap";

string input_dir;
string output_dir;
string input_file;
string output_file;
string snpprofile_file;

string required_param[] =
{
  "input_file",
  "output_file",
  "snpprofile_file"
};
int num_required_param = sizeof(required_param) / sizeof(required_param[0]);

void InitParams()
{
  params.ExpandImports();

  srand(TimerSeconds());
  if (params.Contains("rand_seed"))
    srand(params["rand_seed"].GetInt());

  params.Require(vector<string>(&required_param[0],
                                &required_param[num_required_param]));

  if (params.Contains("input_dir"))
    input_dir = params["input_dir"].GetString();
  if (params.Contains("input_file"))
    input_file = params["input_file"].GetString();
  
  if (params.Contains("output_dir"))
    output_dir = params["output_dir"].GetString();
  if (params.Contains("output_file"))
    output_file = params["output_file"].GetString();
     
  if (params.Contains("snpprofile_file"))
    snpprofile_file = params["snpprofile_file"].GetString();
}


int Main(vector<string> args)
{
  cerr << "Import23Genotype" << endl;
  InitBaseTables();
  params.ImportArgs(vector<string>(args.begin() + 1, args.end()));
  InitParams();
  cerr << endl;

  SNPProfile profile;
  SNPData data;

  {
    istream *in = InFileStream(RelPath(input_dir, snpprofile_file));
    AssertMsg(in, RelPath(input_dir, snpprofile_file));

    profile.ReadBin(*in);
 
    delete in;
    profile.ComputeDerived();
    profile.Validate();
  }

  data.SetProfile(profile);
  data.pops.push_back(SNPData::Pop("pop"));
  
  vector<int8> haplo0, haplo1;
  haplo0.resize(profile.seq_len, 4);
  haplo1.resize(profile.seq_len, 4);

  map<string, int> rs_index;
  for (int i = 0; i < profile.loci.size(); ++i)
    rs_index[profile.loci[i].label] = i;

  istream *in = InFileStream(RelPath(input_dir, input_file));
  AssertMsg(in, RelPath(input_dir, input_file));

  string line;
  vector<string> fields;
  int total = 0, found = 0;
  while (getline(*in, line))
  {
    if (line.empty() || (line[0] == '#'))
      continue;

    SplitFields(line, fields, " \t");
    if (fields.empty())
      continue;
    Assert(fields.size() >= 4);

    map<string, int>::const_iterator find = rs_index.find(fields[0]);
    if (find != rs_index.end())
    {
      int i = find->second;
      Assert(fields[3].length() == 2);
      if (fields[3][0] != '-')
        haplo0[i] = char2base[fields[3][0]];
      if (fields[3][1] != '-')
        haplo1[i] = char2base[fields[3][1]];
      ++found;
    }
    ++total;
  }
  
  delete in;

  cerr << "Mapped " << found << " / " << total << " genotypes" << endl;

  data.AddDiplo(haplo0, haplo1, 0.5, 0);
  data.ComputeDerived();
  data.Validate();

  ostream *out = OutFileStream(RelPath(output_dir, output_file));
  AssertMsg(out, RelPath(output_dir, output_file));

  WriteBinSNPProfileData(*out, data);

  delete out;
  
  cerr << "Done!" << endl;
  return 0;
}
