
#ifndef PHASINGERROR_H
#define PHASINGERROR_H

#include "System.h"
#include "SNPData.h"
#include "HaploModel.h"


class PhasingError
{
public:
  vector<float> x_switch;

  PhasingError() { }

  PhasingError(const SNPData &data, 
               const vector<int8> &x0,
               const HaploModel::Forward &forward0,
               const HaploModel::Backward &backward0,
               const vector<int8> &x1,
               const HaploModel::Forward &forward1,
               const HaploModel::Backward &backward1,
               const vector<uint8> &x_switch_prior)
  {
    Calculate(data, x0, forward0, backward0, x1, forward1, backward1, x_switch_prior);
  }
  
  void Calculate(const SNPData &data, 
                 const vector<int8> &x0,
                 const HaploModel::Forward &forward0,
                 const HaploModel::Backward &backward0,
                 const vector<int8> &x1,
                 const HaploModel::Forward &forward1,
                 const HaploModel::Backward &backward1,
                 const vector<uint8> &x_switch_prior);


  PhasingError(const SNPData &data, 
               const vector<int8> &x0,
               const HaploModel::ViterbiForward &forward0,
               const HaploModel::ViterbiBackward &backward0,
               const vector<int8> &x1,
               const HaploModel::ViterbiForward &forward1,
               const HaploModel::ViterbiBackward &backward1,
               const vector<uint8> &x_switch_prior)
  {
    Calculate(data, x0, forward0, backward0, x1, forward1, backward1, x_switch_prior);
  }
  
  void Calculate(const SNPData &data, 
                 const vector<int8> &x0,
                 const HaploModel::ViterbiForward &forward0,
                 const HaploModel::ViterbiBackward &backward0,
                 const vector<int8> &x1,
                 const HaploModel::ViterbiForward &forward1,
                 const HaploModel::ViterbiBackward &backward1,
                 const vector<uint8> &x_switch_prior);


  float operator[](int index) const { return x_switch[index]; }
  float &operator[](int index) { return x_switch[index]; }

  int size() const { return int(x_switch.size()); }

private:
  static double CalculateSwitchSum(const SNPData &data,
                                   const HaploModel::StateProfile<HaploModel::Prob> &s0,
                                   const HaploModel::StateProfile<HaploModel::Prob> &s1);

  static double CalculateSwitchMax(const SNPData &data,
                                   const HaploModel::StateProfile<HaploModel::Prob> &s0,
                                   const HaploModel::StateProfile<HaploModel::Prob> &s1);
};


#endif
